@@ -1,8 +1,7 @@
 /*****************************************************************************
  * reordertp.c: rebuild an RTP stream from several aggregated links
  *****************************************************************************
- * Copyright (C) 2009, 2011, 2014-2015 VideoLAN
- * $Id$
+ * Copyright (C) 2009, 2011, 2014-2017 VideoLAN
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -55,6 +54,9 @@
 #define DEFAULT_RETX_DELAY 200 /* ms */
 #define MIN_RETX_DELAY 10 /* ms */
 #define DEFAULT_MAX_RETX_BURST 15 /* packets */
+#define RETX_REFRACTORY_TRIGGER 100 /* uncorrected errors */
+#define RETX_REFRACTORY_PERIOD 15000 /* ms */
+#define RETX_REFRACTORY_RESET 3000 /* ms */
 
 /*****************************************************************************
  * Local declarations
@@ -72,6 +74,7 @@ typedef struct input_t
 {
     int i_fd;
     bool b_tcp;
+    bool b_multicast;
     block_t *p_block;
     sockaddr_t peer;
 } input_t;
@@ -83,6 +86,7 @@ static int i_output_fd;
 static input_t *p_inputs = NULL;
 static int i_nb_inputs = 0;
 static int b_udp = 0;
+static int b_redundance = 0;
 
 static block_t *p_first = NULL;
 static block_t **pp_retx = NULL;
@@ -111,12 +115,17 @@ static uint64_t i_retx_delay = DEFAULT_RETX_DELAY * 27000;
 static int i_retx_fd = -1;
 static unsigned int i_max_retx_burst = DEFAULT_MAX_RETX_BURST;
 static int i_last_retx_input = 0;
+static unsigned int i_retx_uncorrected_errors = 0;
+static uint64_t i_retx_uncorrected_errors_expiration = UINT64_MAX;
+static uint16_t i_last_output_seqnum = 0;
+static uint64_t i_retx_refractory_end = 0;
 
 static void usage(void)
 {
-    msg_Raw( NULL, "Usage: reordertp [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-b <buffer length>] [-U] [-g <max gap>] [-j <max jitter>] [-r <# of clock ref>] [-n <max retx burst>] [-x <reorder/retx delay>] [-X <retx URL>] [-m <payload size>] [-R <RTP header>] <src host 1> ... [<src host N>] <dest host>" );
+    msg_Raw( NULL, "Usage: reordertp [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-b <buffer length>] [-U] [-D] [-g <max gap>] [-j <max jitter>] [-r <# of clock ref>] [-n <max retx burst>] [-x <reorder/retx delay>] [-X <retx URL>] [-m <payload size>] [-R <RTP header>] <src host 1> ... [<src host N>] <dest host>" );
     msg_Raw( NULL, "    host format: [<connect addr>[:<connect port>]][@[<bind addr][:<bind port>]]" );
     msg_Raw( NULL, "    -U: strip RTP header" );
+    msg_Raw( NULL, "    -D: input has redundant packets" );
     msg_Raw( NULL, "    -b: buffer length in ms [default 400]" );
     msg_Raw( NULL, "    -g: max gap between two clock references in ms [default 300]" );
     msg_Raw( NULL, "    -j: max jitter in ms [default 150]" );
@@ -211,6 +220,40 @@ static void RetxPacketSent( block_t *p_block )
     for ( i = 0; i < i_nb_retx; i++ )
         if ( pp_retx[i] == p_block )
             pp_retx[i] = NULL;
+
+    uint16_t i_seqnum = p_block->i_seqnum;
+
+    if ( i_retx_refractory_end )
+    {
+        if ( p_block->i_date > i_retx_refractory_end )
+        {
+            msg_Warn( NULL, "now reenabling retx" );
+            i_retx_refractory_end = 0;
+        }
+    }
+    else if ( i_seqnum != i_last_output_seqnum + 1 )
+    {
+        if ( ++i_retx_uncorrected_errors >= RETX_REFRACTORY_TRIGGER )
+        {
+            msg_Warn( NULL, "too many errors, disabling retx for a while" );
+            i_retx_uncorrected_errors = 0;
+            i_retx_refractory_end =
+                p_block->i_date + RETX_REFRACTORY_PERIOD * 27000;
+        }
+        else
+        {
+            i_retx_uncorrected_errors_expiration =
+                p_block->i_date + RETX_REFRACTORY_RESET * 27000;
+        }
+    }
+    else if ( p_block->i_date > i_retx_uncorrected_errors_expiration )
+    {
+        i_retx_uncorrected_errors_expiration = UINT64_MAX;
+        i_retx_uncorrected_errors = 0;
+    }
+
+
+    i_last_output_seqnum = i_seqnum;
 }
 
 static void RetxDereference( block_t *p_block )
@@ -241,7 +284,8 @@ static int RetxGetFd(sockaddr_t **pp_sockaddr)
         i_nb_tries++;
         i_last_retx_input++;
         i_last_retx_input %= i_nb_inputs;
-        if ( p_inputs[i_last_retx_input].peer.so.sa_family != AF_UNSPEC )
+        if ( p_inputs[i_last_retx_input].peer.so.sa_family != AF_UNSPEC &&
+             !p_inputs[i_last_retx_input].b_multicast )
             break;
     }
 
@@ -281,7 +325,9 @@ static void RetxCheck( uint64_t i_current_date )
 
             if ( i_current_seqnum == i_prev_seqnum )
             {
-                msg_Dbg( NULL, "duplicate RTP packet %hu", i_current_seqnum );
+                if ( !b_redundance )
+                    msg_Dbg( NULL, "duplicate RTP packet %hu",
+                             i_current_seqnum );
                 RetxDereference( p_current );
                 free( p_current );
                 continue;
@@ -293,7 +339,8 @@ static void RetxCheck( uint64_t i_current_date )
                                             (i_prev_seqnum + 1)) % POW2_16;
                 sockaddr_t *p_sockaddr;
                 int i_fd;
-                if ( i_nb_packets <= i_max_retx_burst &&
+                if ( !i_retx_refractory_end &&
+                     i_nb_packets <= i_max_retx_burst &&
                      (i_fd = RetxGetFd(&p_sockaddr)) != -1 )
                 {
                     uint8_t p_buffer[RETX_HEADER_SIZE];
@@ -445,18 +492,20 @@ int main( int i_argc, char **pp_argv )
     struct pollfd *pfd = NULL;
     int i_fd;
     bool b_tcp;
+    bool b_multicast = false;
 
 #define ADD_INPUT                                                           \
     p_inputs = realloc( p_inputs, ++i_nb_inputs * sizeof(input_t) );        \
     p_inputs[i_nb_inputs - 1].i_fd = i_fd;                                  \
     p_inputs[i_nb_inputs - 1].b_tcp = b_tcp;                                \
+    p_inputs[i_nb_inputs - 1].b_multicast = b_multicast;                    \
     p_inputs[i_nb_inputs - 1].p_block = NULL;                               \
     p_inputs[i_nb_inputs - 1].peer.so.sa_family = AF_UNSPEC;                \
     pfd = realloc( pfd, i_nb_inputs * sizeof(struct pollfd) );              \
     pfd[i_nb_inputs - 1].fd = i_fd;                                         \
     pfd[i_nb_inputs - 1].events = POLLIN | POLLERR | POLLRDHUP | POLLHUP;
 
-    while ( (c = getopt( i_argc, pp_argv, "i:l:t:b:g:j:r:n:x:X:Um:R:h" )) != -1 )
+    while ( (c = getopt( i_argc, pp_argv, "i:l:t:b:g:j:r:n:x:X:UDm:R:h" )) != -1 )
     {
         switch ( c )
         {
@@ -511,6 +560,10 @@ int main( int i_argc, char **pp_argv )
             b_udp = 1;
             break;
 
+        case 'D':
+            b_redundance = 1;
+            break;
+
         case 'm':
             i_asked_payload_size = strtol( optarg, NULL, 0 );
             break;
@@ -533,8 +586,12 @@ int main( int i_argc, char **pp_argv )
 
     while ( optind < i_argc - 1 )
     {
+        struct opensocket_opt opt;
+        memset(&opt, 0, sizeof(struct opensocket_opt));
+        opt.pb_multicast = &b_multicast;
+
         i_fd = OpenSocket( pp_argv[optind], 0, DEFAULT_PORT, 0, NULL,
-                           &b_tcp, NULL );
+                           &b_tcp, &opt );
         if ( i_fd == -1 )
         {
             msg_Err( NULL, "unable to open input %s\n", pp_argv[optind] );
