@@ -404,7 +404,11 @@ static struct addrinfo *ParseNodeService( char *_psz_string, char **ppsz_end,
         struct in_addr addr;
         if ( inet_aton( psz_node, &addr ) != 0 )
         {
-            struct sockaddr_in *p_sin = malloc( sizeof(struct sockaddr_in) );
+            uint8_t *p_buffer = malloc( sizeof(struct addrinfo) +
+                                        sizeof(struct sockaddr_in) );
+            p_res = (struct addrinfo *)p_buffer;
+            struct sockaddr_in *p_sin =
+                (struct sockaddr_in *)(p_buffer + sizeof(struct addrinfo));
             p_sin->sin_family = AF_INET;
             if ( psz_port != NULL )
                 p_sin->sin_port = ntohs( atoi( psz_port ) );
@@ -412,7 +416,6 @@ static struct addrinfo *ParseNodeService( char *_psz_string, char **ppsz_end,
                 p_sin->sin_port = 0;
             p_sin->sin_addr = addr;
 
-            p_res = malloc( sizeof(struct addrinfo) );
             p_res->ai_family = AF_INET;
             p_res->ai_socktype = SOCK_DGRAM;
             p_res->ai_protocol = 0;
@@ -552,7 +555,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     socklen_t i_sockaddr_len;
     bool b_host = false;
     bool b_raw_packets = false;
-    in_addr_t i_raw_srcaddr = INADDR_ANY; 
+    in_addr_t i_raw_srcaddr = INADDR_ANY;
     int i_raw_srcport = 0;
     char *psz_ifname = NULL;
 #ifdef __FreeBSD__
@@ -566,6 +569,9 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
         pb_tcp = &b_tcp;
     *pb_tcp = false;
 
+    if ( p_opt != NULL && p_opt->pb_multicast != NULL )
+        *p_opt->pb_multicast = false;
+
     psz_token2 = strrchr( psz_arg, ',' );
     if ( psz_token2 )
     {
@@ -777,6 +783,9 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                                       &connect_addr );
                         exit(EXIT_FAILURE);
                     }
+
+                    if ( p_opt != NULL && p_opt->pb_multicast != NULL )
+                        *p_opt->pb_multicast = true;
                 }
                 else
                 #endif
@@ -878,6 +887,9 @@ normal_bind:
                 psz_ifname = NULL;
             }
 #endif
+
+            if ( p_opt != NULL && p_opt->pb_multicast != NULL )
+                *p_opt->pb_multicast = true;
         }
     }
 
