@@ -1,7 +1,7 @@
 /*****************************************************************************
  * util.h: Utils for the multicat suite
  *****************************************************************************
- * Copyright (C) 2009, 2011, 2014-2016 VideoLAN
+ * Copyright (C) 2009, 2011, 2014-2017 VideoLAN
  * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
@@ -23,6 +23,7 @@
 
 #include <netinet/udp.h>
 #include <netinet/ip.h>
+#include <sys/types.h>
 
 #if defined(__APPLE__) || defined(__FreeBSD__)
 #define POLLRDHUP 0
@@ -59,14 +60,12 @@ typedef union
  * Raw udp packet structure with flexible-array payload
  *****************************************************************************/
 struct udprawpkt {
-#if !defined(__APPLE__)
-#if defined(__FreeBSD__)
+#if defined(__APPLE__) || defined(__FreeBSD__)
     struct  ip iph;
 #else
     struct  iphdr iph;
 #endif
     struct  udphdr udph;
-#endif
     uint8_t payload[];
 } __attribute__((packed));
 
@@ -76,6 +75,7 @@ struct udprawpkt {
  *****************************************************************************/
  struct opensocket_opt {
     struct udprawpkt *p_raw_pktheader;
+    bool *pb_multicast;
  };
 
 
